/**
  ******************************************************************************
  * @file      startup_stm32l151xba.s
  * @author    MCD Application Team
  * @brief     STM32L151XBA Devices vector table for GCC toolchain.
  *            This module performs:
  *                - Set the initial SP
  *                - Set the initial PC == Reset_Handler,
  *                - Set the vector table entries with the exceptions ISR address
  *                - Configure the clock system
  *                - Branches to main in the C library (which eventually
  *                  calls main()).
  *            After Reset the Cortex-M3 processor is in Thread mode,
  *            priority is Privileged, and the Stack is set to Main.
  ******************************************************************************
  *
  * @attention
  *
  * Copyright (c) 2017 STMicroelectronics. All rights reserved.
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the 
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */

  .syntax unified
  .cpu cortex-m3
  .fpu softvfp
  .thumb

.global g_pfnVectors
.global Default_Handler

/* start address for the initialization values of the .data section.
defined in linker script */
.word _sidata
/* start address for the .data section. defined in linker script */
.word _sdata
/* end address for the .data section. defined in linker script */
.word _edata
/* start address for the .bss section. defined in linker script */
.word _sbss
/* end address for the .bss section. defined in linker script */
.word _ebss

.equ  BootRAM, 0xF108F85F
/**
 * @brief  This is the code that gets called when the processor first
 *          starts execution following a reset event. Only the absolutely
 *          necessary set is performed, after which the application
 *          supplied main() routine is called.
 * @param  None
 * @retval : None
*/

  .section .text.Reset_Handler
  .weak Reset_Handler
  .type Reset_Handler, %function
Reset_Handler:

/* Copy the data segment initializers from flash to SRAM */
  ldr r0, =_sdata
  ldr r1, =_edata
  ldr r2, =_sidata
  movs r3, #0
  b LoopCopyDataInit

CopyDataInit:
  ldr r4, [r2, r3]
  str r4, [r0, r3]
  adds r3, r3, #4

LoopCopyDataInit:
  adds r4, r0, r3
  cmp r4, r1
  bcc CopyDataInit
  
/* Zero fill the bss segment. */
  ldr r2, =_sbss
  ldr r4, =_ebss
  movs r3, #0
  b LoopFillZerobss

FillZerobss:
  str  r3, [r2]
  adds r2, r2, #4

LoopFillZerobss:
  cmp r2, r4
  bcc FillZerobss

/* Call the clock system intitialization function.*/
    bl  SystemInit
/* Call static constructors */
    bl __libc_init_array
/* Call the application's entry point.*/
  bl main
  bx lr
.size Reset_Handler, .-Reset_Handler

/**
 * @brief  This is the code that gets called when the processor receives an
 *         unexpected interrupt.  This simply enters an infinite loop, preserving
 *         the system state for examination by a debugger.
 *
 * @param  None
 * @retval : None
*/
    .section .text.Default_Handler,"ax",%progbits
Default_Handler:
Infinite_Loop:
  b Infinite_Loop
  .size Default_Handler, .-Default_Handler
/******************************************************************************
*
* The minimal vector table for a Cortex M3.  Note that the proper constructs
* must be placed on this to ensure that it ends up at physical address
* 0x0000.0000.
*
******************************************************************************/
   .section .isr_vector,"a",%progbits
  .type g_pfnVectors, %object
  .size g_pfnVectors, .-g_pfnVectors


g_pfnVectors:
  .word _estack                       /* 1 */
  .word Reset_Handler                 /* 2 */
  .word NMI_Handler                   /* 3 */
  .word HardFault_Handler             /* 4 */
  .word MemManage_Handler             /* 5 */
  .word BusFault_Handler              /* 6 */ 
  .word UsageFault_Handler            /* 7 */
  .word 0                             /* 8 */
  .word 0                             /* 9 */
  .word 0                             /* 10 */ 
  .word 0                             /* 11 */
  .word SVC_Handler                   /* 12 */
  .word DebugMon_Handler              /* 13 */
  .word 0                             /* 14 */
  .word PendSV_Handler                /* 15 */
  .word SysTick_Handler               /* 16 */
  .word WWDG_IRQHandler               /* 17 */
  .word PVD_IRQHandler                /* 18 */ 
  .word TAMPER_STAMP_IRQHandler       /* 19 */
  .word RTC_WKUP_IRQHandler           /* 20 */
  .word FLASH_IRQHandler              /* 21 */
  .word RCC_IRQHandler                /* 22 */
  .word EXTI0_IRQHandler              /* 23 */
  .word EXTI1_IRQHandler              /* 24 */
  .word EXTI2_IRQHandler              /* 25 */
  .word EXTI3_IRQHandler              /* 26 */
  .word EXTI4_IRQHandler              /* 27 */
  .word DMA1_Channel1_IRQHandler      /* 28 */
  .word DMA1_Channel2_IRQHandler      /* 29 */
  .word DMA1_Channel3_IRQHandler      /* 30 */
  .word DMA1_Channel4_IRQHandler      /* 31 */
  .word DMA1_Channel5_IRQHandler      /* 32 */
  .word DMA1_Channel6_IRQHandler      /* 33 */
  .word DMA1_Channel7_IRQHandler      /* 34 */
  .word ADC1_IRQHandler               /* 35 */
  .word USB_HP_IRQHandler             /* 36 */
  .word USB_LP_IRQHandler             /* 37 */
  .word DAC_IRQHandler                /* 38 */
  .word COMP_IRQHandler               /* 39 */
  .word EXTI9_5_IRQHandler            /* 40 */
  .word 0                             /* 41 */
  .word TIM9_IRQHandler               /* 42 */
  .word TIM10_IRQHandler              /* 43 */
  .word TIM11_IRQHandler              /* 44 */
  .word TIM2_IRQHandler               /* 45 */
  .word TIM3_IRQHandler               /* 46 */
  .word TIM4_IRQHandler               /* 47 */
  .word I2C1_EV_IRQHandler            /* 48 */
  .word I2C1_ER_IRQHandler            /* 49 */
  .word I2C2_EV_IRQHandler            /* 50 */
  .word I2C2_ER_IRQHandler            /* 51 */
  .word SPI1_IRQHandler               /* 52 */
  .word SPI2_IRQHandler               /* 53 */
  .word USART1_IRQHandler             /* 54 */
  .word USART2_IRQHandler             /* 55 */
  .word USART3_IRQHandler             /* 56 */
  .word EXTI15_10_IRQHandler          /* 57 */
  .word RTC_Alarm_IRQHandler          /* 58 */
  .word USB_FS_WKUP_IRQHandler        /* 59 */
  .word TIM6_IRQHandler               /* 60 */
  .word TIM7_IRQHandler               /* 61 */
  .word 0                             /* 62 */
  .word 0                             /* 63 */
  .word 0                             /* 64 */
  .word 0                             /* 65 */
  .word 0                             /* 66 */
  .word 0                             /* 67 */
  .word 0                             /* 68 */
  .word 0                             /* 69 */
  .word 0                             /* 70 */
  .word 0                             /* 71 */
  .word 0                             /* 72 */
  .word 0                             /* 73 */
  .word 0                             /* 74 */
  .word 0                             /* 75 */
  .word 0                             /* 76 */
  .word 0                             /* 77 */
  .word 0                             /* 78 */
  .word BootRAM          /* @0x108. This is for boot in RAM mode for STM32L151XBA devices. 79 */

/*******************************************************************************
*
* Provide weak aliases for each Exception handler to the Default_Handler.
* As they are weak aliases, any function with the same name will override
* this definition.
*
*******************************************************************************/

  .weak NMI_Handler
  .thumb_set NMI_Handler,Default_Handler

  .weak HardFault_Handler
  .thumb_set HardFault_Handler,Default_Handler

  .weak MemManage_Handler
  .thumb_set MemManage_Handler,Default_Handler

  .weak BusFault_Handler
  .thumb_set BusFault_Handler,Default_Handler

  .weak UsageFault_Handler
  .thumb_set UsageFault_Handler,Default_Handler

  .weak SVC_Handler
  .thumb_set SVC_Handler,Default_Handler

  .weak DebugMon_Handler
  .thumb_set DebugMon_Handler,Default_Handler

  .weak PendSV_Handler
  .thumb_set PendSV_Handler,Default_Handler

  .weak SysTick_Handler
  .thumb_set SysTick_Handler,Default_Handler

  .weak WWDG_IRQHandler
  .thumb_set WWDG_IRQHandler,Default_Handler

  .weak PVD_IRQHandler
  .thumb_set PVD_IRQHandler,Default_Handler

  .weak TAMPER_STAMP_IRQHandler
  .thumb_set TAMPER_STAMP_IRQHandler,Default_Handler

  .weak RTC_WKUP_IRQHandler
  .thumb_set RTC_WKUP_IRQHandler,Default_Handler

  .weak FLASH_IRQHandler
  .thumb_set FLASH_IRQHandler,Default_Handler

  .weak RCC_IRQHandler
  .thumb_set RCC_IRQHandler,Default_Handler

  .weak EXTI0_IRQHandler
  .thumb_set EXTI0_IRQHandler,Default_Handler

  .weak EXTI1_IRQHandler
  .thumb_set EXTI1_IRQHandler,Default_Handler

  .weak EXTI2_IRQHandler
  .thumb_set EXTI2_IRQHandler,Default_Handler

  .weak EXTI3_IRQHandler
  .thumb_set EXTI3_IRQHandler,Default_Handler

  .weak EXTI4_IRQHandler
  .thumb_set EXTI4_IRQHandler,Default_Handler

  .weak DMA1_Channel1_IRQHandler
  .thumb_set DMA1_Channel1_IRQHandler,Default_Handler

  .weak DMA1_Channel2_IRQHandler
  .thumb_set DMA1_Channel2_IRQHandler,Default_Handler

  .weak DMA1_Channel3_IRQHandler
  .thumb_set DMA1_Channel3_IRQHandler,Default_Handler

  .weak DMA1_Channel4_IRQHandler
  .thumb_set DMA1_Channel4_IRQHandler,Default_Handler

  .weak DMA1_Channel5_IRQHandler
  .thumb_set DMA1_Channel5_IRQHandler,Default_Handler

  .weak DMA1_Channel6_IRQHandler
  .thumb_set DMA1_Channel6_IRQHandler,Default_Handler

  .weak DMA1_Channel7_IRQHandler
  .thumb_set DMA1_Channel7_IRQHandler,Default_Handler

  .weak ADC1_IRQHandler
  .thumb_set ADC1_IRQHandler,Default_Handler

  .weak USB_HP_IRQHandler
  .thumb_set USB_HP_IRQHandler,Default_Handler

  .weak USB_LP_IRQHandler
  .thumb_set USB_LP_IRQHandler,Default_Handler

  .weak DAC_IRQHandler
  .thumb_set DAC_IRQHandler,Default_Handler

  .weak COMP_IRQHandler
  .thumb_set COMP_IRQHandler,Default_Handler

  .weak EXTI9_5_IRQHandler
  .thumb_set EXTI9_5_IRQHandler,Default_Handler

  .weak TIM9_IRQHandler
  .thumb_set TIM9_IRQHandler,Default_Handler

  .weak TIM10_IRQHandler
  .thumb_set TIM10_IRQHandler,Default_Handler

  .weak TIM11_IRQHandler
  .thumb_set TIM11_IRQHandler,Default_Handler

  .weak TIM2_IRQHandler
  .thumb_set TIM2_IRQHandler,Default_Handler

  .weak TIM3_IRQHandler
  .thumb_set TIM3_IRQHandler,Default_Handler

  .weak TIM4_IRQHandler
  .thumb_set TIM4_IRQHandler,Default_Handler

  .weak I2C1_EV_IRQHandler
  .thumb_set I2C1_EV_IRQHandler,Default_Handler

  .weak I2C1_ER_IRQHandler
  .thumb_set I2C1_ER_IRQHandler,Default_Handler

  .weak I2C2_EV_IRQHandler
  .thumb_set I2C2_EV_IRQHandler,Default_Handler

  .weak I2C2_ER_IRQHandler
  .thumb_set I2C2_ER_IRQHandler,Default_Handler

  .weak SPI1_IRQHandler
  .thumb_set SPI1_IRQHandler,Default_Handler

  .weak SPI2_IRQHandler
  .thumb_set SPI2_IRQHandler,Default_Handler

  .weak USART1_IRQHandler
  .thumb_set USART1_IRQHandler,Default_Handler

  .weak USART2_IRQHandler
  .thumb_set USART2_IRQHandler,Default_Handler

  .weak USART3_IRQHandler
  .thumb_set USART3_IRQHandler,Default_Handler

  .weak EXTI15_10_IRQHandler
  .thumb_set EXTI15_10_IRQHandler,Default_Handler

  .weak RTC_Alarm_IRQHandler
  .thumb_set RTC_Alarm_IRQHandler,Default_Handler

  .weak USB_FS_WKUP_IRQHandler
  .thumb_set USB_FS_WKUP_IRQHandler,Default_Handler

  .weak TIM6_IRQHandler
  .thumb_set TIM6_IRQHandler,Default_Handler

  .weak TIM7_IRQHandler
  .thumb_set TIM7_IRQHandler,Default_Handler

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/

